// mainwnd.cpp : implementation of all classes used in Mazeing
// with the exception of CMazeingApp class
//

#include "stdafx.h"
#include "mazeing.h"

#include "mainwnd.h"

#include "globals.h"
#include "myctl3d.h"
#include <string.h>
#include <fcntl.h>
#include <io.h>

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

extern CMazeingApp NEAR theApp;


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// CMainWnd - main window

BEGIN_MESSAGE_MAP(CMainWnd, CFrameWnd)
	//{{AFX_MSG_MAP(CMainWnd)
	ON_WM_SYSCOLORCHANGE()
	ON_COMMAND(IDM_MAZENEW, OnMazeNew)
	ON_WM_PAINT()
	ON_COMMAND(IDM_MAZESHOW, OnMazeShow)
	ON_UPDATE_COMMAND_UI(IDM_MAZESHOW, OnUpdateMazeShow)
	ON_WM_SIZE()
	ON_WM_MOVE()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(IDM_MAZEPRINT, OnPrint)
	ON_WM_SETCURSOR()
	ON_COMMAND(IDM_MAZEAUTOPRINT, OnAutoPrint)
	ON_COMMAND(IDM_MAZECOPY, OnMazeCopy)
	ON_COMMAND(IDM_FILESAVE, OnFileSave)
	ON_COMMAND(IDM_FILELOAD, OnFileLoad)
	ON_UPDATE_COMMAND_UI(IDM_FILESAVE, OnUpdateFileSave)
	ON_UPDATE_COMMAND_UI(IDM_MAZECOPY, OnUpdateMazeCopy)
	ON_UPDATE_COMMAND_UI(IDM_MAZEPRINT, OnUpdateMazePrint)
	ON_COMMAND(IDM_HELPCONTENTS, OnHelpContents)
	ON_COMMAND(IDM_HELPONHELP, OnHelpOnHelp)
	ON_COMMAND(IDM_MAZEPRINTSETUP, OnMazePrintSetup)
	ON_UPDATE_COMMAND_UI(IDM_SHOWTRACK, OnUpdateShowTrack)
	ON_COMMAND(IDM_SHOWTRACK, OnShowTrack)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()




/////////////////////////////////////////////////////////////////////////////
// CMainWnd construction/destruction

CMainWnd::CMainWnd() : PrintDialog(TRUE)
{
	const char *classname ;
	HBRUSH WndBkBrush ;							// window background colour brush 
	char buf[100] ;
    CRect rect ; //, crect;
    
    // pointer so that we can get to the dialog from non member functions
    pPrintDialog = &PrintDialog ;

	// initialize m_pd fields of PrintDialog:
	PrintDialog.m_pd.hwndOwner = m_hWnd;
	PrintDialog.m_pd.Flags |= PD_ENABLESETUPHOOK ;
	PrintDialog.m_pd.lpfnSetupHook = (UINT (CALLBACK *)(HWND, UINT, WPARAM, LPARAM))MakeProcInstance((FARPROC)OpenHook, AfxGetInstanceHandle());
    
	rect.left = GetPrivateProfileInt("Window", "Left", 5, "Mazeing.ini") ;
	rect.top = GetPrivateProfileInt("Window", "Top", 5, "Mazeing.ini") ;
	rect.right = GetPrivateProfileInt("Window", "Right", 441, "Mazeing.ini") ;
	rect.bottom = GetPrivateProfileInt("Window", "Bottom", 472, "Mazeing.ini") ;

	GetPrivateProfileString("Print Margins", "Top", "1.5 cm", buf, 100, "Mazeing.ini") ;
	MTop = buf ;
	GetPrivateProfileString("Print Margins", "Bottom", "1.5 cm", buf, 100, "Mazeing.ini") ;
	MBot = buf ;
	GetPrivateProfileString("Print Margins", "Left", "1.5 cm", buf, 100, "Mazeing.ini") ;
	MLft = buf ;
	GetPrivateProfileString("Print Margins", "Right", "1.5 cm", buf, 100, "Mazeing.ini") ;
    MRgt = buf ;

	WndBkBrush = CreateSolidBrush(GetSysColor(COLOR_WINDOW)) ;

	classname = AfxRegisterWndClass(
			CS_BYTEALIGNWINDOW|CS_DBLCLKS|CS_HREDRAW|CS_VREDRAW,
			DefaultCursor = theApp.LoadStandardCursor(IDC_ARROW),
			WndBkBrush,
			theApp.LoadIcon(AFX_IDI_STD_FRAME)
			);
	
    LoadAccelTable(MAKEINTRESOURCE(IDR_MAINFRAME));

    Create( classname,                  	// Class name or NULL for default
	        "Mazeing",         				// Title bar text
	        WS_OVERLAPPEDWINDOW,            // Window style        
	        rect,							// size & position, use rectDefault for default
	        NULL,                           // parent window or NULL if main window
	        MAKEINTRESOURCE(IDR_MAINFRAME)	// menu name or NULL if no menu
	        );

	// get the maze dimensions
	yd = GetPrivateProfileInt("Maze", "Height", 30, "Mazeing.ini") + 1 ;
	xd = GetPrivateProfileInt("Maze", "Width", 30, "Mazeing.ini") + 1 ;
	if(xd < 5 || xd > 2000 || yd < 5 || yd > 2000 || (long)xd * (long)yd > 60000L) xd = yd = 30 ;
    
	BeepOnComplete = GetPrivateProfileInt("Maze", "BeepOnComplete", 0, "Mazeing.ini") ;
	
	PencilCursor = LoadCursor(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDC_PENCIL)) ;
}




CMainWnd::~CMainWnd()
{
	UnLockArrays() ; FreeArrays() ;
}




/////////////////////////////////////////////////////////////////////////////
// CMainWnd diagnostics

#ifdef _DEBUG
void CMainWnd::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainWnd::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG




/////////////////////////////////////////////////////////////////////////////
// CMainWnd system message handlers


#ifdef SHOWTRACE		// used for debugging only
unsigned char oa[40000] ;
#endif


void CMainWnd::OnPaint()
{
	CPaintDC dc(this) ;		// device context for painting
    CRect rect;

    GetClientRect( rect ) ;
	dc.SetMapMode(MM_ISOTROPIC) ;
	dc.SetWindowExt((xd + 1)<<3, (yd + 1)<<3) ;
    dc.SetViewportExt(rect.right, rect.bottom) ; 
	dc.SetWindowOrg(-8, -8) ;
		    
#ifdef SHOWTRACE		// used for debugging only
	register unsigned int x, y ;
	if(MazeGenMode > 0 && MazeGenMode < MAX_GEN_MODE) {
	    RECT r ;
		LockArrays() ;
		for(x = 0 ; x < xd ; x++)
			for(y = 0 ; y < yd ; y++) 
				if((CELL(x, y) & USED) != oa[(unsigned)x*(unsigned)yd + (unsigned)y]) {
						oa[(unsigned)x*(unsigned)yd+(unsigned)y] = (CELL(x, y) & USED) ;
						r.top = (y<<3) + 2 ;
						r.left = (x<<3) + 2 ;		
						r.bottom = r.top + 4 ;
						r.right = r.left + 4 ;
						dc.InvertRect(&r) ;		
						}
		UnLockArrays() ;
		}
#endif

	if(ValidMaze) {
		CPen *OldPen ;
		CPen MazePen(PS_SOLID, 0, GetSysColor(COLOR_WINDOWTEXT)) ;
		CPen SolutionPen(PS_SOLID, 1, RGB(0, 0, 255)) ;		// BLUE
		CPen TrackPen(PS_SOLID, 1, RGB(255, 0, 0)) ;		// RED

		OldPen = dc.SelectObject(&MazePen) ;

		LockArrays() ;
	    AbortPrint = 0 ;
	    
	    DrawMaze(&dc) ;

		if(TrackLength > 1 && ShowTrack) {
			dc.SelectObject(&TrackPen) ;
			for(int i = 1 ; i < TrackLength ; i++)
				DrawTrackLine(&dc, i) ;
			}
				
		if(ShowSolution) {
			dc.SelectObject(&SolutionPen) ;
			DrawSolution(&dc) ;
			}
			
		dc.SelectObject(OldPen) ;
		UnLockArrays() ;
		}
}



// returns non zero if the mouse on on the track drawn by the user
// sets the global MouseIndex to the position as an index into TPath?[]
// sets global MouseOnTrack if the mouse is somewhere on the track
int SearchTrack(int x, int y, int tracking)
{
	int i ;
	
	// first, have we run off the top or bottom?
	if(y < -1 || y >= yd)
		return (MouseOnTrack = FALSE) ;
	
	// if there is a track, run up it and see if we are on it
	// search is started one back from the end if we are tracking
	for(i = (tracking ? max(0, TrackLength - 2) : 0) ; i < TrackLength ; i++)
		if(TPathX[i] == x && TPathY[i] == y) {
			MouseIndex = i ;
			return (MouseOnTrack = TRUE) ;
			}	

	// is it one cell beyond the track		
	if(TrackLength > 0) {
		if(TPathX[TrackLength-1] == x) {
			if((TPathY[TrackLength-1] == y+1 && !(CELL(x, y+1) & TOP))
					|| (TPathY[TrackLength-1] == y-1 && !(CELL(x, y) & TOP)))
				return (MouseOnTrack = MouseIndex = TrackLength) ;
			}
		else if(TPathY[TrackLength-1] == y && y != -1 && y < yd-1) {
			if((TPathX[TrackLength-1] == x+1 && !(CELL(x+1, y) & LFT))
					|| (TPathX[TrackLength-1] == x-1 && !(CELL(x, y) & LFT)))
				return (MouseOnTrack = MouseIndex = TrackLength) ;
			}
		}
	
	//is it an entrance
	if(TrackLength <= 1 && !tracking)
		if((PathX[0] == x && y == -1)
				|| (PathX[PathLength] == x && PathY[PathLength]+1 == y)) {
			MouseIndex = 0 ;
			return (MouseOnTrack = TRUE) ;
			}
			
	return (MouseOnTrack = FALSE) ;
}




// used to implement the drawing tracks feature
void CMainWnd::OnMouseMove(UINT nFlags, CPoint point)
{
	register int x, y ;
	CPoint p = point ;
	static CPoint OldPoint ;
	static int OldX, OldY ;
	static int tracking = 0 ;
	CClientDC dc(pMainWnd) ;
	CRect rect ;
	
	if(!ValidMaze || !(ShowTrack || TrackLength <=1))  goto DoAFXProcessing ;

    GetClientRect(rect) ;
	dc.SetMapMode(MM_ISOTROPIC) ;
	dc.SetWindowExt((xd + 1) << 3, (yd + 1) << 3) ;
    dc.SetViewportExt(rect.right, rect.bottom) ; 
	dc.SetWindowOrg(-8, -8) ;
		
	// convert screen position into maze cell reference                          
	dc.DPtoLP(&p) ;
	x = p.x < 0 ? -1 : p.x / 8 ;	// special case if negative
	y = p.y < 0 ? -1 : p.y / 8 ;	// special case if negative 
	
	// this is a kludge!
	// pretend that we are  out of the grid if the mouse is close to getting
	// out of the client area.  This is necessary because if the mouse pointer enters
	// the non client area set to a pencil the #*$@ MFC will not let me change it!!!
	if(p.y <= -6) y = -2 ;
	if(y == yd - 1 && (p.y % 8) >= 6) y = -2 ;


	if(!(nFlags & MK_LBUTTON)) tracking = 0 ;

	if(MouseX == x && MouseY == y && MouseFlags == nFlags) goto DoAFXProcessing ;
	MouseX = x ; MouseY = y ; MouseFlags = nFlags ;
	MouseOnTrack = 0 ;


	// if on track then process accordingly
	// NOTE:  The mouse pointer is set in OnSetCursor() according to MouseOnTrack
	if(SearchTrack(x, y, tracking)) {
		if(nFlags & MK_LBUTTON) {
			tracking = 1 ;
			ProcessMouseClick(&dc) ;
			}
		else
			tracking = 0 ;
		}
	else {
		if((nFlags & MK_LBUTTON) && tracking) {
			// correct if the mouse has drifted off track while holding down the right button
			CPoint NewClientPoint = point ;
			if(OldX != x && OldY == y) {
				x = OldX ;
				NewClientPoint.x = OldPoint.x ;
				}
			else if(OldX == x && OldY != y) {
				y = OldY ;
				NewClientPoint.y = OldPoint.y ;
				}
			else if(SearchTrack(OldX, y, 1)) {
				x = OldX ;
				NewClientPoint.x = OldPoint.x ;
				}
			else if(SearchTrack(x, OldY, 1)) {
				y = OldY ;
				NewClientPoint.y = OldPoint.y ;
				}
			else {
				x = OldX ;  y = OldY ;
			    NewClientPoint = OldPoint ;
			    }
			point = NewClientPoint ;
			ClientToScreen(&NewClientPoint) ;
			SetCursorPos(NewClientPoint.x, NewClientPoint.y) ;
			MouseOnTrack = 1 ;
			}
		else {
			tracking = 0 ;
			}
		}
			
 // skip to here if not processing this mouse movement
 DoAFXProcessing:
	// save the mouse position and the previous incase we need to control the mouse
	if(tracking) {
		OldX = x ; OldY = y ;
		OldPoint = point ;
		}
	else
		OldX = OldY = OldPoint.x = OldPoint.y = -1 ;
	CFrameWnd::OnMouseMove(nFlags, point);
}




// used to implement the drawing tracks feature
void ProcessMouseClick(CDC *dc)
{

	// are we at the beginning of a track ?
	// this complex if statement basically says:
	//		IF at entrance and first time clicked OR if already selected one entrance
	//		and the user is clicking on the other entrance,
	// I am sure that the statement could be simplified but what the heck! it works!
    if((MouseIndex <= 0 && TrackLength <= 1)
    		|| ((MouseIndex <= 1 && TrackLength <= 1) && ((TPathY[0] == -1 && MouseY == yd -1)
    			|| (TPathY[0] == yd-1 && MouseY ==  -1)))) {
    	TPathX[0] = MouseX ;
    	TPathY[0] = MouseY ;
		TrackLength = 1 ;  TrackTime = GetTickCount() ;
		ShowTrack = 0 ;
		}   
	// or are we adding to it ?
	else if(MouseIndex == TrackLength) {
		TPathX[TrackLength] = MouseX ;	
		TPathY[TrackLength] = MouseY ;
			{
			CPen *OldPen, Pen(PS_SOLID, 1, RGB(255,0,0)) ;
			
			OldPen = dc->SelectObject(&Pen) ;
			DrawTrackLine(dc, TrackLength) ;
			dc->SelectObject(OldPen) ;
			}
		// detect if the user has finished
		if((MouseY == yd-1 || MouseY == -1) && TrackTime != -1) {
			char buf[200] ;
			TrackTime = (GetTickCount() - TrackTime) / 1000 ;
			wsprintf(buf, "Congratulations\r\nYou have finished the maze.\r\n\nTotal time was %ld minutes %ld seconds.", TrackTime/60L, TrackTime%60L) ;
			AfxMessageBox(buf) ;
			TrackTime = -1 ;
			} 
		ShowTrack = 1 ;
		TrackLength++ ;
		}
	// finally, are we in the middle and want to delete
	else {
		CPen *OldPen, Pen(PS_SOLID, 1, GetSysColor(COLOR_WINDOW)) ;
		CPen SolutionPen(PS_SOLID, 1, RGB(0, 0, 255)) ;		// BLUE

		OldPen = dc->SelectObject(&Pen) ;
		while(MouseIndex < TrackLength - 1)
			if(ValidMaze)
				DrawTrackLine(dc, --TrackLength) ;
	
		if(ShowSolution) {
			dc->SelectObject(&SolutionPen) ;
			DrawSolution(dc) ;
			}
			
		dc->SelectObject(OldPen) ;
		}
}


		
// used to implement the drawing tracks feature
void CMainWnd::OnLButtonDown(UINT nFlags, CPoint point)
{
	
	if(MouseOnTrack) {
		CClientDC dc(pMainWnd) ;
		CRect rect ;
		
	    GetClientRect(rect) ;
		dc.SetMapMode(MM_ISOTROPIC) ;
		dc.SetWindowExt((xd + 1) << 3, (yd + 1) << 3) ;
	    dc.SetViewportExt(rect.right, rect.bottom) ; 
		dc.SetWindowOrg(-8, -8) ;
		
		ProcessMouseClick(&dc) ;
		}
	
	CFrameWnd::OnLButtonDown(nFlags, point);
}





// set the mouse pointer depending if we are on the track
// for some reason MFC ignores this if the mouse is not in
// the client area.
BOOL CMainWnd::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message)
{
	static HCURSOR OldC = 0 ;
	
	if(pWnd == this)
		if(nHitTest == HTCLIENT && MouseOnTrack) {
			OldC = SetCursor(PencilCursor) ;
			return FALSE ;
			}
		else if(OldC) {
			SetCursor(OldC) ;
			OldC = 0 ;
			return FALSE ;
			}
	
	
	return CFrameWnd::OnSetCursor(pWnd, nHitTest, message);
}




// save the window size and position if changed
void CMainWnd::OnSize(UINT nType, int cx, int cy)
{
	CFrameWnd::OnSize(nType, cx, cy);
	
	WriteProfileSize(this) ;	
}




// save the window size and position if changed
void CMainWnd::OnMove(int x, int y)
{
	CFrameWnd::OnMove(x, y);

	WriteProfileSize(this) ;	
}




void CMainWnd::OnSysColorChange()
{
	CFrameWnd::OnSysColorChange();
	Ctl3dColorChange() ;		// inform the 3-D stuff about the change
	DeleteObject((const void *)SetClassWord(m_hWnd, GCW_HBRBACKGROUND, unsigned short(CreateSolidBrush(GetSysColor(COLOR_WINDOW))))) ;
	Invalidate() ;
	
}




/////////////////////////////////////////////////////////////////////////////////////////
// CMainWnd - Menu handlers

void CMainWnd::OnMazeNew()
{
	CMazeNew dlg ;
	char buf[10] ;
	
#ifdef SHOWTRACE		// used for debugging only
	for(int i = 0 ; i < 30000 ; i++) oa[i] = 0 ;
#endif
	
	dlg.m_BeepOnComplete = BeepOnComplete ;
	dlg.m_Height = yd - 1 ;
	dlg.m_Width = xd - 1 ;
	if(dlg.DoModal() == IDOK) {
		if(yd - 1 != dlg.m_Height || xd - 1 != dlg.m_Width) {
		
			yd = dlg.m_Height + 1 ;
			xd = dlg.m_Width + 1;
	
			wsprintf(buf, "%i", yd - 1) ;
			WritePrivateProfileString("Maze", "Height", buf, "Mazeing.ini") ;
			wsprintf(buf, "%i", xd - 1) ;
			WritePrivateProfileString("Maze", "Width", buf, "Mazeing.ini") ;

			FreeArrays() ;
			}
	    
		if(dlg.m_BeepOnComplete != BeepOnComplete) {
			BeepOnComplete = dlg.m_BeepOnComplete ;			
			WritePrivateProfileString("Maze", "BeepOnComplete", BeepOnComplete ? "1":"0", "Mazeing.ini") ;
			}
		
	    ShowSolution = 0 ;
		MazeGenMode = 0 ;
		ValidMaze = 0 ;
		Invalidate(TRUE) ;
		}
}




// show the solution
void CMainWnd::OnMazeShow()
{
	CClientDC dc(pMainWnd) ;
	CRect rect ;
	CPen *OldPen ;
	CPen SolutionPen(PS_SOLID, 1, !ShowSolution?RGB(0,0,255):GetSysColor(COLOR_WINDOW)) ;
	CPen TrackPen(PS_SOLID, 1, RGB(255, 0, 0)) ;		// RED
		
	ShowSolution = !ShowSolution ;

    GetClientRect( rect ) ;
	dc.SetMapMode(MM_ISOTROPIC) ;
	dc.SetWindowExt((xd + 1) << 3, (yd + 1) << 3) ;
    dc.SetViewportExt(rect.right, rect.bottom) ; 
	dc.SetWindowOrg(-8, -8) ;
	
	OldPen = dc.SelectObject(&SolutionPen) ;
	DrawSolution(&dc) ;

	if(!ShowSolution && ShowTrack) {
		dc.SelectObject(&TrackPen) ;
		for(int i = 1 ; i < TrackLength ; i++)
			DrawTrackLine(&dc, i) ;
		}

	dc.SelectObject(OldPen) ;
}





void CMainWnd::OnShowTrack()
{
	CClientDC dc(pMainWnd) ;
	CRect rect ;
	CPen *OldPen ;
	CPen SolutionPen(PS_SOLID, 1, RGB(0,0,255)) ;
	CPen TrackPen(PS_SOLID, 1, !ShowTrack?RGB(255,0,0):GetSysColor(COLOR_WINDOW)) ;		// RED
		
	ShowTrack = !ShowTrack ;

    GetClientRect( rect ) ;
	dc.SetMapMode(MM_ISOTROPIC) ;
	dc.SetWindowExt((xd + 1) << 3, (yd + 1) << 3) ;
    dc.SetViewportExt(rect.right, rect.bottom) ; 
	dc.SetWindowOrg(-8, -8) ;
	
	OldPen = dc.SelectObject(&TrackPen) ;
	for(int i = 1 ; i < TrackLength ; i++)
		DrawTrackLine(&dc, i) ;

	if(!ShowTrack && ShowSolution) {
		dc.SelectObject(&SolutionPen) ;
		DrawSolution(&dc) ;
		}
		
	dc.SelectObject(OldPen) ;
}





void CMainWnd::OnPrint()
{
	CMyPrintDialog MyPrintDialog ;
	char buf[30] ;

	if(!ValidMaze) return ;
	
	PrintDialog.GetDefaults() ;
	if(::GlobalLock(PrintDialog.m_pd.hDevNames) == NULL) {
		NotifyMsg("Cannot obtain default printer details\r\n\nSetup a default printer using Control Panel") ;
		return ;
		}
	else
		::GlobalUnlock(PrintDialog.m_pd.hDevNames) ;

	MyPrintDialog.m_PrintCopies = 1 ;
	MyPrintDialog.m_PrintSolution = ShowSolution ;
	MyPrintDialog.m_PrintTrack = (TrackLength > 1 && ShowTrack) ;
	wsprintf(buf, "%d x %d", xd-1, yd-1) ;
	if(MyPrintDialog.DoModal() == IDOK)
		DoPrintMaze(MyPrintDialog.m_PrintCopies, MyPrintDialog.m_PrintSolution, MyPrintDialog.m_PrintTrack, buf) ;
}





void CMainWnd::OnAutoPrint()
{
	CAutoPrintDialog AutoPrintDialog ;
	
	PrintDialog.GetDefaults() ;
	if(::GlobalLock(PrintDialog.m_pd.hDevNames) == NULL) {
		NotifyMsg("Cannot obtain default printer details\r\n\nSetup a default printer using Control Panel") ;
		return ;
		}
	else
		::GlobalUnlock(PrintDialog.m_pd.hDevNames) ;

	AutoPrintDialog.m_Mazes = AutoPrintDialog.m_Blank = 1 ;
	AutoPrintDialog.m_Solution = 0 ;

	if(AutoPrintDialog.DoModal() == IDOK 
						&& AutoPrintDialog.m_Mazes 
						&& (AutoPrintDialog.m_Blank || AutoPrintDialog.m_Solution)) {
		// we are off!!  Setup the global variables used by the auto printing
		AutoPrintMazes = AutoPrintDialog.m_Mazes ;
		AutoPrintBlank = AutoPrintDialog.m_Blank ;
		AutoPrintSolution = AutoPrintDialog.m_Solution ;
		AutoPrintCount = 0 ;
	    // trigger the generation of a new maze
	    ShowSolution = 0 ;
		MazeGenMode = 0 ;
		ValidMaze = 0 ;
		Invalidate(TRUE) ;
        // the maze generation routine (MazeOnIdle()) will automatically print the maze
        // in the the last processing steps.  If there are more mazes to print MazeOnIdle()
        // will also retrigger itself to generate another maze.
		}
}




// copy to the clipboard
void CMainWnd::OnMazeCopy()
{
	CMetaFileDC dc ;
	CPen MazePen(PS_SOLID, 0, GetSysColor(COLOR_WINDOWTEXT)) ;
	CPen SolutionPen(PS_SOLID, 1, RGB(0, 0, 255)) ;		// BLUE
	CPen TrackPen(PS_SOLID, 1, RGB(255, 0, 0)) ;		// RED
	GLOBALHANDLE hGMem ;
	LPMETAFILEPICT lpMFP ;
	HMETAFILE hmf ;
	
	if(!ValidMaze) return ;
	
	BeginWaitCursor() ;
	
	// draw the metafile
	dc.Create() ;
	dc.SetWindowExt((xd + 1)<<3, (yd + 1)<<3) ;
	dc.SetWindowOrg(-8, -8) ;
	
	dc.SelectObject(&MazePen) ;

	LockArrays() ;
    AbortPrint = 0 ;
	    
    DrawMaze(&dc) ;

	if(TrackLength > 1 && ShowTrack) {
		dc.SelectObject(&TrackPen) ;
		for(int i = 1 ; i < TrackLength ; i++)
			DrawTrackLine(&dc, i) ;
		}
				
	if(ShowSolution) {
		dc.SelectObject(&SolutionPen) ;
		DrawSolution(&dc) ;
		}
			
	UnLockArrays() ;
	hmf = dc.Close() ;

	// setup the metafile clipboard structure
	hGMem = GlobalAlloc(GHND, (DWORD)sizeof(METAFILEPICT)) ;
	lpMFP = (LPMETAFILEPICT)GlobalLock(hGMem) ;
	lpMFP->mm = MM_ISOTROPIC ;
	lpMFP->xExt = -((xd + 1)<<3) ;	
	lpMFP->yExt = -((yd + 1)<<3) ;
	lpMFP->hMF = hmf ;
	
	// finally place the metafile into the clipboard
	GlobalUnlock(hGMem) ;
	OpenClipboard() ;
	EmptyClipboard() ;
	SetClipboardData(CF_METAFILEPICT, hGMem) ;
	CloseClipboard() ;
	
	EndWaitCursor();	
}




// header string verifies that a file is a Mazeing saved maze
static char MazeFileID[] = "Mazeing V1.1 file" ;

void CMainWnd::OnFileSave()
{
	HFILE hF ;
	OFSTRUCT OF ;
	long TmpL ;
	int r ;
	static CFileDialog dlg(FALSE,
						   "maz",
						   "saved.maz",
						   OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_ENABLEHOOK,
						   " Mazes (*.maz) | *.maz | All Files (*.*) | *.* ||",
						   this) ;    
	
	dlg.m_ofn.lpfnHook = (UINT (CALLBACK *)(HWND, UINT, WPARAM, LPARAM))MakeProcInstance((FARPROC)OpenHook, AfxGetInstanceHandle());
	dlg.m_ofn.lpstrTitle = "Save Maze" ;
	
	r = dlg.DoModal() ;
	FreeProcInstance((FARPROC)dlg.m_ofn.lpfnHook) ;
	if(r != IDOK) return ;
	hF = OpenFile(dlg.GetPathName(), &OF, OF_CREATE | OF_WRITE | OF_SHARE_EXCLUSIVE) ;
	if(hF == HFILE_ERROR) goto SaveError ;
	LockArrays() ;
		
	if(_lwrite(hF, MazeFileID, sizeof(MazeFileID)) != sizeof(MazeFileID)) goto SaveError ;
	if(_lwrite(hF, &xd, sizeof(xd)) != sizeof(xd)) goto SaveError ; 
	if(_lwrite(hF, &yd, sizeof(yd)) != sizeof(yd)) goto SaveError ; 
	if(_lwrite(hF, Cell, (unsigned)(xd+1)*(unsigned)(yd+1)) != (unsigned)(xd+1)*(unsigned)(yd+1)) goto SaveError ; 
	if(_lwrite(hF, &PathLength, sizeof(PathLength)) != sizeof(PathLength)) goto SaveError ; 
	if(_lwrite(hF, PathX, (PathLength+1)*sizeof(*PathX)) != (unsigned)(PathLength+1)*sizeof(*PathX)) goto SaveError ; 
	if(_lwrite(hF, PathY, (PathLength+1)*sizeof(*PathY)) != (unsigned)(PathLength+1)*sizeof(*PathY)) goto SaveError ; 
	if(_lwrite(hF, &TrackLength, sizeof(TrackLength)) != sizeof(TrackLength)) goto SaveError ; 
	if(_lwrite(hF, TPathX, TrackLength*sizeof(*TPathX)) != (unsigned)TrackLength*sizeof(*TPathX)) goto SaveError ; 
	if(_lwrite(hF, TPathY, TrackLength*sizeof(*TPathY)) != (unsigned)TrackLength*sizeof(*TPathY)) goto SaveError ; 
	if(_lwrite(hF, &ShowSolution, sizeof(ShowSolution)) != sizeof(ShowSolution)) goto SaveError ;
	if(_lwrite(hF, &ShowTrack, sizeof(ShowTrack)) != sizeof(ShowTrack)) goto SaveError ;
	if(TrackTime != -1)
		TmpL = GetTickCount() -  TrackTime ;
	else
		TmpL = -1 ;
	if(_lwrite(hF, &TmpL, sizeof(TmpL)) != sizeof(TmpL)) goto SaveError ;
	 
	_lclose(hF) ; 
	UnLockArrays() ;
	return ;

 SaveError:
	_lclose(hF) ; 
	UnLockArrays() ;
 	NotifyMsg("Error creating or writing to %s\r\nMaze not saved", dlg.GetPathName()) ;
}




void CMainWnd::OnFileLoad()
{
	char LoadedFileID[sizeof(MazeFileID)] ;
	HFILE hF ;
	OFSTRUCT OF ;
	long TmpL ;
	int r ;
	static CFileDialog dlg(TRUE,
					"maz",
					"saved.maz",
					OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_ENABLEHOOK,
					" Mazes (*.maz) | *.maz | All Files (*.*) | *.* ||",
					this) ;    
	dlg.m_ofn.lpfnHook = (UINT (CALLBACK *)(HWND, UINT, WPARAM, LPARAM))MakeProcInstance((FARPROC)OpenHook, AfxGetInstanceHandle());
	dlg.m_ofn.lpstrTitle = "Load Maze" ;

	r = dlg.DoModal() ;
	FreeProcInstance((FARPROC)dlg.m_ofn.lpfnHook) ;
	if(r != IDOK) return ;
	
	hF = OpenFile(dlg.GetPathName(), &OF, OF_READ | OF_SHARE_COMPAT) ;
	if(hF == HFILE_ERROR) goto LoadError ;
	
	if(_lread(hF, LoadedFileID, sizeof(MazeFileID)) != sizeof(MazeFileID)) goto LoadError ;
	if(_fstrcmp(LoadedFileID, MazeFileID) != 0) goto LoadError ;
	FreeArrays() ;
	ShowTrack = ValidMaze = 0 ;
	if(_lread(hF, &xd, sizeof(xd)) != sizeof(xd)) goto LoadError ; 
	if(_lread(hF, &yd, sizeof(yd)) != sizeof(yd)) goto LoadError ; 
	if(xd < 5 || xd > 2000 || yd < 5 || yd > 2000 || (long)xd * (long)yd > 60000L) {
		xd = yd = 30 ;
		goto LoadError ;
		}
	if(!AllocateArrays()) goto LoadError ;
	LockArrays() ;
	if(_lread(hF, Cell, (unsigned)(xd+1)*(unsigned)(yd+1)) != (unsigned)(xd+1)*(unsigned)(yd+1)) goto LoadError ; 
	if(_lread(hF, &PathLength, sizeof(PathLength)) != sizeof(PathLength)) goto LoadError ; 
	if(_lread(hF, PathX, (PathLength+1)*sizeof(*PathX)) != (unsigned)(PathLength+1)*sizeof(*PathX)) goto LoadError ; 
	if(_lread(hF, PathY, (PathLength+1)*sizeof(*PathY)) != (unsigned)(PathLength+1)*sizeof(*PathY)) goto LoadError ; 
	if(_lread(hF, &TrackLength, sizeof(TrackLength)) != sizeof(TrackLength)) goto LoadError ; 
	if(_lread(hF, TPathX, TrackLength*sizeof(*TPathX)) != (unsigned)TrackLength*sizeof(*TPathX)) goto LoadError ; 
	if(_lread(hF, TPathY, TrackLength*sizeof(*TPathY)) != (unsigned)TrackLength*sizeof(*TPathY)) goto LoadError ; 
	if(_lread(hF, &ShowSolution, sizeof(ShowSolution)) != sizeof(ShowSolution)) goto LoadError ;
	if(_lread(hF, &ShowTrack, sizeof(ShowTrack)) != sizeof(ShowTrack)) goto LoadError ;
	if(_lread(hF, &TmpL, sizeof(TmpL)) != sizeof(TmpL)) goto LoadError ;
	if(TmpL != -1)
		TrackTime = GetTickCount() -  TmpL ;
	else
		TrackTime = -1 ;
	_lclose(hF) ; 
	UnLockArrays() ;
	ValidMaze = 1 ;
	Invalidate() ;
	return ;

 LoadError:
	_lclose(hF) ; 
	UnLockArrays() ;
 	NotifyMsg("Error reading %s\r\nMaze not loaded", dlg.GetPathName()) ;
}




void CMainWnd::OnHelpContents()
{
	theApp.WinHelp(0L, HELP_CONTENTS) ;
}




void CMainWnd::OnHelpOnHelp()
{
	theApp.WinHelp(0L, HELP_HELPONHELP) ;
}



void CMainWnd::OnMazePrintSetup()
{
	pPrintDialog->m_pd.Flags &= ~PD_RETURNDEFAULT ;
	if(pPrintDialog->DoModal() != IDOK) return ;
}




/////////////////////////////////////////////////////////////////////////////////////////
// CMainWnd - User interface update handlers (eg enable menu entries)

void CMainWnd::OnUpdateMazeShow(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(ValidMaze) ;
	pCmdUI->SetCheck(ShowSolution && ValidMaze) ;
}




void CMainWnd::OnUpdateFileSave(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(ValidMaze) ;
}




void CMainWnd::OnUpdateMazeCopy(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(ValidMaze) ;
	
}



void CMainWnd::OnUpdateMazePrint(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(ValidMaze) ;
	
}




void CMainWnd::OnUpdateShowTrack(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(ValidMaze && TrackLength > 1) ;
	pCmdUI->SetCheck(ShowTrack && ValidMaze && TrackLength > 1) ;
	
}




/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// CMazeNew dialog

CMazeNew::CMazeNew(CWnd* pParent /*=NULL*/)
	: CDialog(CMazeNew::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMazeNew)
	m_Height = 0;
	m_Width = 0;
	m_BeepOnComplete = FALSE;
	//}}AFX_DATA_INIT
}



void CMazeNew::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMazeNew)
	DDX_Control(pDX, IDD_MAZEWIDTH, m_ctlWidth);
	DDX_Control(pDX, IDC_MAZEHEIGHT, m_ctlHeight);
	DDX_Text(pDX, IDC_MAZEHEIGHT, m_Height);
	DDX_Text(pDX, IDD_MAZEWIDTH, m_Width);
	DDX_Check(pDX, IDC_BEEPFINISHED, m_BeepOnComplete);
	//}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CMazeNew, CDialog)
	//{{AFX_MSG_MAP(CMazeNew)
	ON_BN_CLICKED(ID_MAZENEWHELP, OnClickedHelp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



// overridden this to check that a valid maze size has been entered
void CMazeNew::OnOK()
{
	const int maxlen = 10 ;
	char h[maxlen], w[maxlen] ;
	      
	h[m_ctlHeight.GetLine(0, h, maxlen)] = 0 ;
	w[m_ctlWidth.GetLine(0, w, maxlen)] = 0 ;
	if(atol(h) < 5L || atol(h) > 2000L || atol(w) < 5L || atol(w) > 2000L || atol(h) * atol(w) > 60000L) {
		NotifyMsg("Invalid maze dimensions\r\n\nThe height and width must be between 5 and 2000\r\nwith a total  maze size of less than 60000 cells") ;
		if(atol(w) < 5L || atol(w) > 2000L) { m_ctlWidth.SetFocus() ; m_ctlWidth.SetSel(0, -1) ; }
		if(atol(h) < 5L || atol(h) > 2000L) { m_ctlHeight.SetFocus() ; m_ctlHeight.SetSel(0, -1) ; }
		return ;
		}
	CDialog::OnOK();
}




void CMazeNew::OnClickedHelp()
{
	theApp.WinHelp(101) ;
}





/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// CMyPrintDialog dialog

CMyPrintDialog::CMyPrintDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CMyPrintDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMyPrintDialog)
	m_PrintSolution = FALSE;
	m_PrintTrack = FALSE;
	m_PrintCopies = 0;
	//}}AFX_DATA_INIT
}




void CMyPrintDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMyPrintDialog)
	DDX_Control(pDX, IDC_PRINTCOPIES, m_ctlPrintCopies);
	DDX_Control(pDX, IDC_PRINTNAME, m_PrintName);
	DDX_Control(pDX, IDC_PRINTTRACK, m_ctlPrintTrack);
	DDX_Control(pDX, IDC_PRINTSOLUTION, m_ctlPrintSolution);
	DDX_Check(pDX, IDC_PRINTSOLUTION, m_PrintSolution);
	DDX_Check(pDX, IDC_PRINTTRACK, m_PrintTrack);
	DDX_Text(pDX, IDC_PRINTCOPIES, m_PrintCopies);
	//}}AFX_DATA_MAP

}




BEGIN_MESSAGE_MAP(CMyPrintDialog, CDialog)
	//{{AFX_MSG_MAP(CMyPrintDialog)
	ON_BN_CLICKED(IDC_PRINTPRINTER, OnClickedPrintPrinter)
	ON_BN_CLICKED(IDC_PRINTMARGINS, OnClickedPrintMargins)
	ON_BN_CLICKED(ID_MAZEPRINTHELP, OnClickedHelp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()




// setup the print dialog box (printer name, etc)
BOOL CMyPrintDialog::OnInitDialog()
{
	CDialog::OnInitDialog() ;

	m_PrintName.SetWindowText(pPrintDialog->GetDeviceName() 
							+ CString("\r\non ") + pPrintDialog->GetPortName()) ;
	m_ctlPrintTrack.EnableWindow(TrackLength > 1) ;
	m_ctlPrintCopies.SetFocus() ;
	m_ctlPrintCopies.SetSel(0, -1) ;
	
	return FALSE ;
}




// show the printer setup common dialog
void CMyPrintDialog::OnClickedPrintPrinter()
{
	pPrintDialog->m_pd.Flags &= ~PD_RETURNDEFAULT ;
	if(pPrintDialog->DoModal() != IDOK) return ;
	
	m_PrintName.SetWindowText(pPrintDialog->GetDeviceName() 
							+ CString("\r\non ") + pPrintDialog->GetPortName()) ;
}




// show the page margins dialog
void CMyPrintDialog::OnClickedPrintMargins()
{
	CPrintMargins dlg(this) ;
	
	dlg.m_sMarginTop = MTop ;
	dlg.m_sMarginBottom = MBot ;
	dlg.m_sMarginLeft = MLft ;
	dlg.m_sMarginRight = MRgt ;
	
	if(dlg.DoModal() == IDOK) {
		MTop = dlg.m_sMarginTop ;
		MBot = dlg.m_sMarginBottom ;
		MLft = dlg.m_sMarginLeft ;
		MRgt = dlg.m_sMarginRight ;
		WriteProfileMargins() ;
		}
}



void CMyPrintDialog::OnClickedHelp()
{
	theApp.WinHelp(102) ;
}




/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// CAutoPrintDialog dialog

CAutoPrintDialog::CAutoPrintDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CAutoPrintDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CAutoPrintDialog)
	m_Blank = 0;
	m_Mazes = 0;
	m_Solution = 0;
	//}}AFX_DATA_INIT
}



void CAutoPrintDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAutoPrintDialog)
	DDX_Control(pDX, IDC_APRINTMAZES, m_ctlMazes);
	DDX_Control(pDX, IDC_APRINTNAME, m_PrintName);
	DDX_Text(pDX, IDC_APRINTBLANK, m_Blank);
	DDX_Text(pDX, IDC_APRINTMAZES, m_Mazes);
	DDX_Text(pDX, IDC_APRINTSOLVED, m_Solution);
	//}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CAutoPrintDialog, CDialog)
	//{{AFX_MSG_MAP(CAutoPrintDialog)
	ON_BN_CLICKED(IDC_APRINTPRINTER, OnClickedPrintPrinter)
	ON_BN_CLICKED(IDC_APRINTMARGINS, OnClickedPrintMargins)
	ON_BN_CLICKED(IDC_APRINTHELP, OnClickedHelp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()




// setup the print dialog box (printer name, etc)
BOOL CAutoPrintDialog::OnInitDialog()
{
	CDialog::OnInitDialog() ;
	
	m_PrintName.SetWindowText(pPrintDialog->GetDeviceName() 
							+ CString("\r\non ") + pPrintDialog->GetPortName()) ;
	m_ctlMazes.SetFocus() ;
	m_ctlMazes.SetSel(0, -1) ;
	
	return FALSE ;
}




// show the printer setup common dialog
void CAutoPrintDialog::OnClickedPrintPrinter()
{
	pPrintDialog->m_pd.Flags &= ~PD_RETURNDEFAULT ;
	if(pPrintDialog->DoModal() != IDOK) return ;
	
	m_PrintName.SetWindowText(pPrintDialog->GetDeviceName() 
							+ CString("\r\non ") + pPrintDialog->GetPortName()) ;
}




// show the page margins dialog
void CAutoPrintDialog::OnClickedPrintMargins()
{
	CPrintMargins dlg(this) ;
	
	dlg.m_sMarginTop = MTop ;
	dlg.m_sMarginBottom = MBot ;
	dlg.m_sMarginLeft = MLft ;
	dlg.m_sMarginRight = MRgt ;
	
	if(dlg.DoModal() == IDOK) {
		MTop = dlg.m_sMarginTop ;
		MBot = dlg.m_sMarginBottom ;
		MLft = dlg.m_sMarginLeft ;
		MRgt = dlg.m_sMarginRight ;
		WriteProfileMargins() ;
		}
}




void CAutoPrintDialog::OnClickedHelp()
{
	theApp.WinHelp(103) ;
}





/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// CPrintMargins dialog

CPrintMargins::CPrintMargins(CWnd* pParent /*=NULL*/)
	: CDialog(CPrintMargins::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPrintMargins)
	m_sMarginBottom = "";
	m_sMarginLeft = "";
	m_sMarginRight = "";
	m_sMarginTop = "";
	//}}AFX_DATA_INIT
}



void CPrintMargins::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPrintMargins)
	DDX_Text(pDX, IDC_MARGINBOTTOM, m_sMarginBottom);
	DDX_Text(pDX, IDC_MARGINLEFT, m_sMarginLeft);
	DDX_Text(pDX, IDC_MARGINRIGHT, m_sMarginRight);
	DDX_Text(pDX, IDC_MARGINTOP, m_sMarginTop);
	//}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CPrintMargins, CDialog)
	//{{AFX_MSG_MAP(CPrintMargins)
	ON_BN_CLICKED(ID_PRINTMARGINSHELP, OnClickedHelp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



void CPrintMargins::OnClickedHelp()
{
	theApp.WinHelp(104) ;
}




/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// CPrintCancelDlg dialog

CPrintCancelDlg::CPrintCancelDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPrintCancelDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPrintCancelDlg)
	//}}AFX_DATA_INIT
}



void CPrintCancelDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPrintCancelDlg)
	DDX_Control(pDX, IDC_PRINTCANCELNAME, m_PrinterName);
	DDX_Control(pDX, IDC_PRINTCANCELMSG, m_Msg);
	//}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CPrintCancelDlg, CDialog)
	//{{AFX_MSG_MAP(CPrintCancelDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



// if the user clicked the cancel button
void CPrintCancelDlg::OnCancel()
{
	AbortPrint = 1;
	AutoPrintMazes = 0 ;
	MazeGenMode = MAX_GEN_MODE + 1 ;
	CDialog::OnCancel();
}



/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// CCalcCancel dialog

CCalcCancel::CCalcCancel(CWnd* pParent /*=NULL*/)
	: CDialog(CCalcCancel::IDD, pParent)
{
	//{{AFX_DATA_INIT(CCalcCancel)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}



void CCalcCancel::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCalcCancel)
	DDX_Control(pDX, IDC_CALC, m_Img);
	DDX_Control(pDX, IDC_CALCCANCELMSG, m_Msg);
	//}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CCalcCancel, CDialog)
	//{{AFX_MSG_MAP(CCalcCancel)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()




// if the user clicked the cancel button
void CCalcCancel::OnCancel()
{
	AbortPrint = 1;
	AutoPrintMazes = 0 ;
	MazeGenMode = MAX_GEN_MODE + 1 ;
	CDialog::OnCancel();
}




BOOL CCalcCancel::OnInitDialog()
{
	CDialog::OnInitDialog() ;
	ThisIcon = 0 ;

	return TRUE ;
}



// animate the hourglass
// every time this member is called it will move the hourglass through
// one step in the animate cycle
void CCalcCancel::Animate()
{
	CBitmap Img ;
    CDC memDC ;
    RECT ImgSize, ImgPosit ;
	CClientDC dc(this) ;
	
	m_Img.GetWindowRect(&ImgPosit) ;		// get the position of the image
	ScreenToClient(&ImgPosit) ;
    	
	ImgSize.left = ImgSize.top = 0 ;
	ImgSize.right = ImgPosit.right - ImgPosit.left ;  
	ImgSize.bottom = ImgPosit.bottom - ImgPosit.top ;

	// set up an in memory bitmap    	
    memDC.CreateCompatibleDC(&dc) ;
    Img.CreateCompatibleBitmap(&dc, ImgSize.right, ImgSize.bottom) ;
    memDC.SelectObject(&Img) ;

    // paint the background colour
    ::FillRect(memDC.m_hDC, &ImgSize, (HBRUSH)::SendMessage(m_hWnd, WM_CTLCOLOR, (unsigned)memDC.m_hDC, MAKELONG(m_Img.m_hWnd, CTLCOLOR_STATIC))) ;

	// get the icon and paint it
    // NOTE all icons must have consecutive numbers after IDC_CALC0 (130)
    //      the id NBR_OF_CALC_ICONS is defined in the resources
    //		force a recompile of this file if NBR_OF_CALC_ICONS is changed
	if(ThisIcon < NBR_OF_CALC_ICONS)
		memDC.DrawIcon(0, 0, ((CStatic*)(CalcCancel.GetDlgItem(IDC_CALC0 + ThisIcon)))->GetIcon()) ;
	else
		memDC.DrawIcon(0, 0, ((CStatic*)(CalcCancel.GetDlgItem(IDC_CALC0 + NBR_OF_CALC_ICONS - 1)))->GetIcon()) ;

    // this routine causes a pause at the end of the sequence
	if(ThisIcon < NBR_OF_CALC_ICONS + 4)
		ThisIcon++ ;
	else
	    ThisIcon = 0 ;
	    
    // draw the lot onto the dialog box
    dc.BitBlt(ImgPosit.left, ImgPosit.top, ImgSize.right, ImgSize.bottom, &memDC, 0, 0, SRCCOPY) ;
}




/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
// Utility Subroutines
//

void AbortMsg(char *szFormat, ...)
{
	char szBuffer[256] ;
	char *pArguments ;
	
	pArguments = (char *)&szFormat + sizeof(szFormat) ;
	vsprintf(szBuffer, szFormat, pArguments) ;
	(void)AfxMessageBox(szBuffer, MB_OK|MB_ICONSTOP) ;
	//(void)pMainWnd->SendMessage(WM_CLOSE) ;
	//(void)TerminateApp(NULL, NO_UAE_BOX) ;
}



void NotifyMsg(char *szFormat, ...)
{
	char szBuffer[256] ;
	char *pArguments ;
	
	pArguments = (char *)&szFormat + sizeof(szFormat) ;
	vsprintf(szBuffer, szFormat, pArguments) ;
	(void)AfxMessageBox(szBuffer, MB_OK|MB_ICONEXCLAMATION) ;
}




void WriteProfileSize(CWnd *wnd)
{
	char buf[10] ;
    CRect rect;
    
	if(!wnd->IsIconic() && !wnd->IsZoomed()) {
	    wnd->GetWindowRect( rect ) ;
		
		wsprintf(buf, "%u", rect.left) ;
		WritePrivateProfileString("Window", "Left", buf, "Mazeing.ini") ;
		wsprintf(buf, "%u", rect.right) ;
		WritePrivateProfileString("Window", "Right", buf, "Mazeing.ini") ;
		wsprintf(buf, "%u", rect.top) ;
		WritePrivateProfileString("Window", "Top", buf, "Mazeing.ini") ;
		wsprintf(buf, "%u", rect.bottom) ;
		WritePrivateProfileString("Window", "Bottom", buf, "Mazeing.ini") ;
		}
}




void WriteProfileMargins()
{
	WritePrivateProfileString("Print Margins", "Left", MLft, "Mazeing.ini") ;
	WritePrivateProfileString("Print Margins", "Right", MRgt, "Mazeing.ini") ;
	WritePrivateProfileString("Print Margins", "Top", MTop, "Mazeing.ini") ;
	WritePrivateProfileString("Print Margins", "Bottom", MBot, "Mazeing.ini") ;
}
